package de.ullisroboterseite.urstinywebdb;

import java.io.*;
import java.net.*;
import org.json.simple.*;

public class UrsTinyWebDBServer extends Thread {
	UrsTinyWebDataBase dataBase;
	ServerSocket listenSocket;

	UrsTinyWebDBServer(ServerSocket listenSocket, UrsTinyWebDataBase dataBase) {
		this.dataBase = dataBase;
		this.listenSocket = listenSocket;
	}

	@Override
	public void run() {
		try {
			Socket connectionSocket = null;
			while (true) {
				try {
					connectionSocket = listenSocket.accept();
					connectionSocket.setSoTimeout(500);
					byte[] inpBuffer = new byte[2048];

					InputStream stream = connectionSocket.getInputStream();
					int bytes = stream.read(inpBuffer);

					try {
						Thread.sleep(300);
						bytes += stream.read(inpBuffer, bytes, inpBuffer.length - bytes);
					} catch (Exception e) {
					}

					String request = new String(inpBuffer);
					request = request.substring(0, bytes);
					CheckAndRespond(request, connectionSocket);
				} catch (Exception e) {
					System.out.println("exception: " + e.toString());
				} finally {
					connectionSocket.close();
				}
			} // while
		} catch (Exception e) {
		}
	}

	private void CheckAndRespond(String request, Socket socket) {
		String payload = "";
		int lastLinePos = request.indexOf("\r\n\r\n");

		if (lastLinePos > -1) {
			payload = request.substring(lastLinePos + 4);
		}

		if (request.startsWith("POST /storeavalue HTTP/1.1")) {
			handleDBStoreValueRequest(payload, socket);
			return;
		}
		if (request.startsWith("POST /getvalue HTTP/1.1")) {
			handleDBGetValueRequest(payload, socket);
			return;
		}
		if (request.startsWith("POST /getall HTTP/1.1") || request.startsWith("GET /getall HTTP/1.1")) {
			handleDBGetAllRequest(socket);
			return;
		}
		String url = request.split("\r")[0];
		sendNotFound("'" + url + "'", socket);
	}

	private void handleDBGetAllRequest(Socket socket) {
		OutputStream out = null;
		try {
			String payload = dataBase.getAll();

			byte[] payloadbytes = payload.getBytes("UTF-8");

			String http = "";
			http += "HTTP/1.1 200 OK\r\n";
			http += "Connection: close\r\n";
			http += "Content-Type: application/jsonrequest; charset=utf-8\r\n";
			http += "Content-Length: " + payloadbytes.length + "\r\n";
			http += "\r\n";

			byte[] output = http.getBytes("UTF-8");
			out = socket.getOutputStream();
			out.write(output);
			out.write(payloadbytes);
			out.close();
		} catch (Exception e) {
			System.out.println(e.toString());
		} finally {
			try {
				out.close();
			} catch (Exception e1) {
			}
		}
	}

	private void handleDBGetValueRequest(String payload, Socket socket) {
		boolean hasTag = false;
		String key = "";
		String fmt = "";

		// Query-Parameter isolieren
		String[] data = payload.split("&");

		for (String s : data) {
			try {

				if (s.startsWith("tag=")) {
					key = java.net.URLDecoder.decode(s.substring(4), "UTF-8");
					hasTag = true;
					continue;
				}
				if (s.startsWith("fmt=")) {
					fmt = java.net.URLDecoder.decode(s.substring(4), "UTF-8");
					continue;
				}
			} catch (Exception e) {
				System.out.println(e.toString());
			}
		}

		// Query-Parameter vollstndig?
		if (!(hasTag)) {
			sendBadRequest("'getvalue' query parameter(s) missing", socket);
			return;
		}

		String EmptyValue = "";
		String value = dataBase.getValue(key);
		if (value == null) {
			value = EmptyValue;
		}

		if (fmt.equals("html")) {
			sendHtmlResponse("VALUE", key, value, socket);
		} else {
			sendDBResponse("VALUE", key, value, socket);
		}
		System.out.println("Delivered: " + key + "=" + value);
	}

	private void handleDBStoreValueRequest(String payload, Socket socket) {
		boolean hasTag = false;
		boolean hasValue = false;
		String key = "";
		String value = "";
		String fmt = "";

		// Query-Parameter isolieren
		String[] data = payload.split("&");

		for (String s : data) {
			try {

				if (s.startsWith("tag=")) {
					key = java.net.URLDecoder.decode(s.substring(4), "UTF-8");
					hasTag = true;
					continue;
				}
				if (s.startsWith("value=")) {
					value = java.net.URLDecoder.decode(s.substring(6), "UTF-8");
					hasValue = true;
					continue;
				}
				if (s.startsWith("fmt=")) {
					fmt = java.net.URLDecoder.decode(s.substring(4), "UTF-8");
					continue;
				}
			} catch (Exception e) {
				System.out.println(e.toString());
			}
		}

		// Query-Parameter vollstndig?
		if (!(hasTag && hasValue)) {
			sendBadRequest("'storeavalue' query parameter(s) missing", socket);
			return;
		}

		String action = "";
		if (value.contentEquals("\"*delete*\"")) {
			action = "DELETED";
			if (dataBase.deleteValue(key))
				action = "DELETED";
			else
				action = "DEL ERROR";

		} else {
			action = "STORED";
			dataBase.storeValue(key, value);
		}

		if (fmt.equals("html")) {
			sendHtmlResponse(action, key, value, socket);
		} else {
			sendDBResponse(action, key, value, socket);
		}
		System.out.println("Stored: " + key + "=" + value);
	}

	@SuppressWarnings("unchecked")
	JSONArray toJSONArray(String actionCode, String tag, String value) {
		JSONArray jList = new JSONArray();
		jList.add(actionCode);
		jList.add(tag);
		jList.add(value);
		return jList;
	}

	private void sendDBResponse(String actionCode, String tag, String value, Socket socket) {
		JSONArray jList = toJSONArray(actionCode, tag, value);

		String payloadJson = jList.toString();
		String http = "";
		http += "HTTP/1.1 200 OK\r\n";
		http += "Connection: close\r\n";
		http += "Content-Type: application/jsonrequest; charset=utf-8\r\n";
		http += "\r\n";
		http += payloadJson;

		OutputStream out = null;
		try {
			byte[] output = http.getBytes("UTF-8");
			out = socket.getOutputStream();
			out.write(output);
			out.close();
		} catch (Exception e) {
			System.out.println(e.toString());
		} finally {
			try {
				out.close();
			} catch (Exception e1) {
			}
		}
	}

	private void sendHtmlResponse(String actionCode, String tag, String value, Socket socket) {
		JSONArray jList = toJSONArray(actionCode, tag, value);

		String PayloadJson = jList.toString();

		try {
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
			try {
				out.write("HTTP/1.1 200 OK\r\n");
				out.write("Connection: close\r\n");
				out.write("Content-Type: Text/html; charset=utf-8\r\n");
				out.write("\r\n");
				out.write("<html><body>Action: " + PayloadJson + "</body></html>\r\n");
			} catch (Exception e) {
				System.out.println(e.toString());
			} finally {
				try {
					out.close();
				} catch (Exception e1) {
				}
			}
		} catch (Exception e) {
		}
	}

	private void sendNotFound(String resource, Socket socket) {
		System.out.println("Resource not found: '" + resource + "'");
		try {
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
			try {
				out.write("Http/1.1 404 Not Found\r\n");
				out.write("Connection: close\r\n");
				out.write("Content-Type: Text/html\r\n");
				out.write("\r\n");
				out.write("<html><body>Not Found: " + resource + "</body></html>\r\n");
			} catch (Exception e) {
				System.out.println(e.toString());
			} finally {
				try {
					out.close();
				} catch (Exception e1) {
				}
			}
		} catch (Exception e) {
		}
	}

	private void sendBadRequest(String message, Socket socket) {
		System.out.println("Bad Request: '" + message + "'");
		try {
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
			try {
				out.write("Http/1.1 400 Bad Request\r\n");
				out.write("Connection: close\r\n");
				out.write("Content-Type: Text/html\r\n");
				out.write("\r\n");
				out.write("<html><body>Bad Request: " + message + "</body></html>\r\n");
			} catch (Exception e) {
				System.out.println(e.toString());
			} finally {
				try {
					out.close();
				} catch (Exception e1) {
				}
			}
		} catch (Exception e) {
		}
	}
}